/*

*/
package scene.models2;
import scene.*;
import scene.primitives.*;

/**
   k   number of lines of longitude
   n+1 number of lines of latitude (there is always at least one)

   r   radius of the base of the cone
   h   height of the cone
*/
public class ConeModel extends Model
{
   public ConeModel( )
   {
      this(3, 2, 1, 1);
   }


   public ConeModel(int k, int n, double radius, double h)
   {
      super();

      // Create the cone's geometry.

      // An array of vertices to be used to create geometric primitives.
      Vertex[][] v = new Vertex[n+1][k];

      // Create all the vertices along the cone wall.
      for (int j = 0; j < k; j++)
      {
         double c = Math.cos(j*(2.0*Math.PI)/k);
         double s = Math.sin(j*(2.0*Math.PI)/k);
         for (int i = 0; i <= n; i++)
         {
            double slantRadius = ((i+1)/(double)(n+1)) * radius;
            v[i][j] = new Vertex( slantRadius*c, h-(i+1)*h/(n+1), slantRadius*s );
         }
      }
      // Vertex at the center of the top.
      Vertex    topCenter = new Vertex(0.0,   h, 0.0);
      // Vertex at the center of the bottom.
      Vertex bottomCenter = new Vertex(0.0, 0.0, 0.0);

      // Create all the horizontal circles of latitude around the cone wall.
      for (int i = 0; i <= n; i++)
      {
         Primitive loop = new LineLoop();
         for (int j = 0; j < k; j++)
         {
            loop.addVertex( v[i][j] );
         }
         this.addPrimitive( loop );
      }

      // Create two line fans.
      Primitive fan1 = new LineFan(); // Line fan from apex to base.
      Primitive fan2 = new LineFan(); // Line fan at the base.
      fan1.addVertex( topCenter );
      fan2.addVertex( bottomCenter );
      for (int j = 0; j < k; j++)
      {
         fan1.addVertex( v[n][j] );
         fan2.addVertex( v[n][j] );
      }
      this.addPrimitive( fan1 );
      this.addPrimitive( fan2 );
   }
}//ConeModel