/*

*/
package scene.models2;
import scene.*;
import scene.primitives.*;

/**
                  v[3]
                   +-----------------+ v[2]
                  /|                /|
                /  |              /  |
              /    |            /    |
            /      |          /      |
      v[4] +-----------------+ v[5]  |
           |       |         |       |               y
           |       |         |       |               |
           |       |         |       |               |
           |  v[0] +---------|-------+ v[1]          |
           |      /          |      /                |
           |    /            |    /                  +----. x
           |  /              |  /                   /
           |/                |/                    /
           +-----------------+                    /
          v[7]              v[6]                 z

   xs the size of the cube along the x-axis
   ys the size of the cube along the y-axis
   zs the size of the cube along the z-axis
*/
public class CubeModel extends Model
{
   public CubeModel( )
   {
      this(1, 1, 1);
   }


   public CubeModel(double xs, double ys, double zs)
   {
      super();

      // Create the cube's geometry.
      Vertex[] v = new Vertex[8];
      v[0] = new Vertex(0,    0,    0);
      v[1] = new Vertex(0+xs, 0,    0);
      v[2] = new Vertex(0+xs, 0+ys, 0);
      v[3] = new Vertex(0,    0+ys, 0);
      v[4] = new Vertex(0,    0+ys, 0+zs);
      v[5] = new Vertex(0+xs, 0+ys, 0+zs);
      v[6] = new Vertex(0+xs, 0,    0+zs);
      v[7] = new Vertex(0,    0,    0+zs);

      // Create one line loop.
      Primitive loop = new LineLoop( v );
      this.addPrimitive( loop );

      // Create four line segments.
      this.addPrimitive( new LineSegment(v[0], v[3]) );
      this.addPrimitive( new LineSegment(v[7], v[4]) );
      this.addPrimitive( new LineSegment(v[1], v[6]) );
      this.addPrimitive( new LineSegment(v[2], v[5]) );
   }
}//CubeModel