/*

*/
package scene.models2;
import scene.*;
import scene.primitives.*;

/**
   k   number of lines of longitude
   n+2 number of lines of latitude (there is always at least two)

   r   radius of the cylinder
   h   height of the cylinder
*/
public class CylinderModel extends Model
{
   public CylinderModel( )
   {
      this(3, 2, 1, 1);
   }


   public CylinderModel(int k, int n, double radius, double h)
   {
      super();

      // Create the cylinder's geometry.

      // An array of vertices to be used to create geometric primitives.
      Vertex[][] v = new Vertex[n+2][k];

      // Create all the vertices along the cylinder wall.
      for (int j = 0; j < k; j++)
      {
         double c = Math.cos(j*(2.0*Math.PI)/k);
         double s = Math.sin(j*(2.0*Math.PI)/k);
         for (int i = 0; i <= n+1; i++)
         {
            v[i][j] = new Vertex( radius*c, h-i*((2.0*h)/(n+1)), radius*s );
         }
      }

      // Vertex at the center of the top.
      Vertex    topCenter = new Vertex(0.0,  h, 0.0);
      // Vertex at the center of the bottom.
      Vertex bottomCenter = new Vertex(0.0, -h, 0.0);

      // Create all the horizontal circles of latitude around the cylinder wall.
      for (int i = 0; i <= n+1; i++)
      {
         Primitive loop = new LineLoop();
         for (int j = 0; j < k; j++)
         {
            loop.addVertex( v[i][j] );
         }
         this.addPrimitive( loop );
      }

      // Create two line fans.
      Primitive fan1 = new LineFan(); // Line fan at the top
      Primitive fan2 = new LineFan(); // Line fan at the base.
      fan1.addVertex( topCenter );
      fan2.addVertex( bottomCenter );
      for (int j = 0; j < k; j++)
      {
         fan1.addVertex( v[0  ][j] );
         fan2.addVertex( v[n+1][j] );
      }
      this.addPrimitive( fan1 );
      this.addPrimitive( fan2 );

      // Create the vertical lines from the top to the bottom.
      for (int j = 0; j < k; j++)
      {
         this.addPrimitive( new LineSegment(v[0][j], v[n+1][j]) );
      }
   }
}//CylinderModel