/*

*/
package scene.models2;
import scene.*;
import scene.primitives.*;

/**
   k   number of lines of longitude
   n+2 number of lines of latitude (there is always at least two)
   r1  radius of the base of the frustum
   r2  radius of the top of the frustum
   h   height of the frustum
*/
public class FrustumCircularModel extends Model
{
   public FrustumCircularModel( )
   {
      this(3, 1, 0.5, 1.0, 1.0);
   }


   public FrustumCircularModel(int k, int n, double r1, double r2, double h)
   {
      super();

      // Create the frustum's geometry.

      // An array of vertices to be used to create geometric primitives.
      Vertex[][] v = new Vertex[n+2][k];

      // Create all the vertices along the frustum wall.
      for (int j = 0; j < k; j++)
      {
         double c = Math.cos(j*(2.0*Math.PI)/k);
         double s = Math.sin(j*(2.0*Math.PI)/k);
         for (int i = 0; i <= n+1; i++)
         {
            double slantRadius = (i/(double)(n+1)) * r1 + ((n+1-i)/(double)(n+1)) * r2;
            v[i][j] = new Vertex( slantRadius*c, h-(i*h)/(n+1), slantRadius*s );
         }
      }

      // Vertex at the center of the top.
      Vertex    topCenter = new Vertex(0.0,   h, 0.0);
      // Vertex at the center of the bottom.
      Vertex bottomCenter = new Vertex(0.0, 0.0, 0.0);

      // Create all the horizontal circles of latitude around the frustum wall.
      for (int i = 0; i <= n+1; i++)
      {
         Primitive loop = new LineLoop();
         for (int j = 0; j < k; j++)
         {
            loop.addVertex( v[i][j] );
         }
         this.addPrimitive( loop );
      }

      // Create two line fans.
      Primitive fan1 = new LineFan(); // Line fan at the top
      Primitive fan2 = new LineFan(); // Line fan at the base.
      fan1.addVertex( topCenter );
      fan2.addVertex( bottomCenter );
      for (int j = 0; j < k; j++)
      {
         fan1.addVertex( v[0  ][j] );
         fan2.addVertex( v[n+1][j] );
      }
      this.addPrimitive( fan1 );
      this.addPrimitive( fan2 );

      // Create the slant lines from the top to the bottom.
      for (int j = 0; j < k; j++)
      {
         this.addPrimitive( new LineSegment(v[0][j], v[n+1][j]) );
      }
   }
}//FrustumCircularModel