/*

*/
package scene.models2;
import scene.*;
import scene.primitives.*;

/**
   Two constructors that mimic the projFrustum() and projPerspective()
   metods from the Camera class.
*/
public class FrustumRectagularModel extends Model
{
   public FrustumRectagularModel( )
   {
      this(-0.25, 0.25, -0.25, 0.25, 0.25, 1);
   }


   public FrustumRectagularModel(double left, double right, double bottom, double top, double near, double far)
   {
      super();

      Vertex v1 = new Vertex(left,  top,    -near);
      Vertex v2 = new Vertex(right, top,    -near);
      Vertex v3 = new Vertex(right, bottom, -near);
      Vertex v4 = new Vertex(left,  bottom, -near);
      Vertex v5 = new Vertex(left,  top,    -far);
      Vertex v6 = new Vertex(right, top,    -far);
      Vertex v7 = new Vertex(right, bottom, -far);
      Vertex v8 = new Vertex(left,  bottom, -far);

      // near face
      Primitive nearFace = new LineLoop();
      nearFace.addVertex( v1 );
      nearFace.addVertex( v2 );
      nearFace.addVertex( v3 );
      nearFace.addVertex( v4 );
      this.addPrimitive( nearFace );
      // far face
      Primitive farFace = new LineLoop();
      farFace.addVertex( v5 );
      farFace.addVertex( v6 );
      farFace.addVertex( v7 );
      farFace.addVertex( v8 );
      this.addPrimitive( nearFace );
      // lines from front to back
      this.addPrimitive( new LineSegment(v1, v5) );
      this.addPrimitive( new LineSegment(v2, v6) );
      this.addPrimitive( new LineSegment(v3, v7) );
      this.addPrimitive( new LineSegment(v4, v8) );
   }


   public FrustumRectagularModel(double fovy, double aspect, double near, double far)
   {
      super();

      double top    =  near * Math.tan((Math.PI/180.0)*fovy/2.0);
      double bottom = -top;
      double right  =  top * aspect;
      double left   = -top * aspect;

      Vertex v1 = new Vertex(left,  top,    -near);
      Vertex v2 = new Vertex(right, top,    -near);
      Vertex v3 = new Vertex(right, bottom, -near);
      Vertex v4 = new Vertex(left,  bottom, -near);
      Vertex v5 = new Vertex(left,  top,    -far);
      Vertex v6 = new Vertex(right, top,    -far);
      Vertex v7 = new Vertex(right, bottom, -far);
      Vertex v8 = new Vertex(left,  bottom, -far);

      // near face
      Primitive nearFace = new LineLoop();
      nearFace.addVertex( v1 );
      nearFace.addVertex( v2 );
      nearFace.addVertex( v3 );
      nearFace.addVertex( v4 );
      this.addPrimitive( nearFace );
      // far face
      Primitive farFace = new LineLoop();
      farFace.addVertex( v5 );
      farFace.addVertex( v6 );
      farFace.addVertex( v7 );
      farFace.addVertex( v8 );
      this.addPrimitive( nearFace );
      // lines from front to back
      this.addPrimitive( new LineSegment(v1, v5) );
      this.addPrimitive( new LineSegment(v2, v6) );
      this.addPrimitive( new LineSegment(v3, v7) );
      this.addPrimitive( new LineSegment(v4, v8) );
   }
}//FrustumRectagularModel