/*

*/
package scene.models2;
import scene.*;
import scene.primitives.*;

/**
   This is a regular octahedron with side length 1.
   Its center is at the orgin and the center plane is the xz-plane.
   The 5th and 6th vertices are on the y-axis.

   http://en.wikipedia.org/wiki/Regular_octahedron
*/
public class OctahedronModel extends Model
{
   public OctahedronModel()
   {
      super();

      // Create the octahedron's geometry.
      double sqrt3 = Math.sqrt(3.0);
      double sqrt2 = Math.sqrt(2.0);
      Vertex[] v = new Vertex[6];
      v[0] = new Vertex( 0.5, 0,  0.5);  // four vertices around the center plane
      v[1] = new Vertex(-0.5, 0,  0.5);
      v[2] = new Vertex(-0.5, 0, -0.5);
      v[3] = new Vertex( 0.5, 0, -0.5);
      v[4] = new Vertex( 0,  1/sqrt2, 0);   // vertex at the top
      v[5] = new Vertex( 0, -1/sqrt2, 0);   // vertex at the bottom

      // Create two line fans and a line loop
      Primitive fan1 = new LineFan();  // Line fan from the top
      Primitive fan2 = new LineFan();  // Line fan from the bottom
      Primitive loop = new LineLoop(); // Loop around the center plane
      fan1.addVertex( v[4] );
      fan2.addVertex( v[5] );
      for (int j = 0; j < 4; j++)
      {
         fan1.addVertex( v[j] );
         fan2.addVertex( v[j] );
         loop.addVertex( v[j] );
      }
      this.addPrimitive( fan1 );
      this.addPrimitive( fan2 );
      this.addPrimitive( loop );
   }
}//OctahedronModel