/*

*/
package scene.models2;
import scene.*;
import scene.primitives.*;

/**
   k   number of lines of longitude
   n+1 number of lines of latitude (there is always at least one)
<p>
   r radius of the sphere
*/
public class SphereModel extends Model
{
   public SphereModel( )
   {
      this(6, 6, 1);
   }


   public SphereModel(int k, int n, double radius)
   {
      super();

      // Create the sphere's geometry.

      // An array of vertices to be used to create geometric primitives.
      Vertex[][] v = new Vertex[n+1][k];

      // Create all the vertices.
      for (int j = 0; j < k; j++)
      {
         double c1 = Math.cos(j*(2.0*Math.PI)/k);
         double s1 = Math.sin(j*(2.0*Math.PI)/k);
         for (int i = 0; i <= n; i++)
         {
            double c2 = Math.cos((i+1)*Math.PI/(n+2));
            double s2 = Math.sin((i+1)*Math.PI/(n+2));
            v[i][j] = new Vertex(radius*s2*c1,
                                 radius*c2,
                                 radius*s2*s1 );
         }
      }
      // Vertex at the center of the top.
      Vertex    topPole = new Vertex(0.0,  radius, 0.0);
      // Vertex at the center of the bottom.
      Vertex bottomPole = new Vertex(0.0, -radius, 0.0);

      // Create all the horizontal circles of latitude around the sphere.
      for (int i = 0; i <= n; i++)
      {
         Primitive loop = new LineLoop();
         for (int j = 0; j < k; j++)
         {
            loop.addVertex( v[i][j] );
         }
         this.addPrimitive( loop );
      }

      // Create two line fans.
      Primitive fan1 = new LineFan(); // Line fan at the top
      Primitive fan2 = new LineFan(); // Line fan at the base.
      fan1.addVertex( topPole );
      fan2.addVertex( bottomPole );
      for (int j = 0; j < k; j++)
      {
         fan1.addVertex( v[0][j] );
         fan2.addVertex( v[n][j] );
      }
      this.addPrimitive( fan1 );
      this.addPrimitive( fan2 );

      // Create the vertical half-circles of longitude.
      for (int j = 0; j < k; j++)
      {
         Primitive lineStrip = new LineStrip();
         // Create the circular arc from the top to the bottom.line fans
         for (int i = 0; i < n; i++)
         {
            lineStrip.addVertex( v[i][j] );
         }
         this.addPrimitive( lineStrip );
      }
   }
}//SphereModel