/*

*/
package scene.models2;
import scene.*;
import scene.primitives.*;

/**
   This is a regular tetrahedron with side length 1.
   One face is in the xz-plane, with its center at the orgin.
   The 4th vertex in on the positive y-axis.

   http://en.wikipedia.org/wiki/Tetrahedron


                     (1, 0, -1/sqrt(2)) (0, 1, 1/sqrt(2)) edge length 2

                     (sqrt(2), 0, -1)   (0, sqrt(2), 1)   edge length 2*sqrt(2)

                     ( 0, 0, 0)  ( 1, 1, 0)  ( 0, 1, 1)  ( 1, 0, 1)  edge length sqrt(2)
        Tetrahedron: ( 1,-1,-1)  (-1, 1,-1)  ( 1, 1, 1)  (-1,-1, 1)  edge length 2*sqrt(2)
   Dual Tetrahedron: (-1, 1, 1)  ( 1,-1, 1)  (-1,-1,-1)  ( 1, 1,-1)
*/
public class TetrahedronModel extends Model
{
   public TetrahedronModel()
   {
      super();

      // Create the tetrahedron's geometry.
      double sqrt3 = Math.sqrt(3.0);
      double sqrt2 = Math.sqrt(2.0);
      Vertex[] v = new Vertex[4];
      v[0] = new Vertex( 1/sqrt3,     0,  0);    // three vertices around the bottom face
      v[1] = new Vertex(-1/(2*sqrt3), 0,  0.5);
      v[2] = new Vertex(-1/(2*sqrt3), 0, -0.5);
      v[3] = new Vertex( 0, sqrt2/sqrt3,  0);    // vertex at the top

      // Create a line fan and a line loop
      Primitive fan  = new LineFan();  // Line fan from the top
      Primitive loop = new LineLoop(); // Loop around the bottom face
      fan.addVertex( v[3] );
      for (int j = 0; j < 3; j++)
      {
          fan.addVertex( v[j] );
         loop.addVertex( v[j] );
      }
      this.addPrimitive( fan );
      this.addPrimitive( loop );
   }
}//TetrahedronModel