/*

*/
package scene.models3;
import scene.*;
import scene.primitives.*;

import java.awt.Color;

/**
   An x and y axis in the xy-plane, along with "tick marks".
*/
public class Axes2DModel extends Model
{
   /**
      Left and right end points for the x-axis.
      Bottom and top end points for the y-axis.
      Number of evenly spaced tick marks on the x-axis.
      Number of evenly spaced tick marks on the y-axis.
      Default color of black.
   */
   public Axes2DModel(double xMin, double xMax, double yMin, double yMax,
                      int xMarks, int yMarks)
   {
      this(xMin, xMax, yMin, yMax, 0.0, xMarks, yMarks, Color.black);
   }


   /**
      Left and right end points for the x-axis.
      Bottom and top end points for the y-axis.
      Number of evenly spaced tick marks on the x-axis.
      Number of evenly spaced tick marks on the y-axis.
      Use the given Color c.
   */
   public Axes2DModel(double xMin, double xMax, double yMin, double yMax,
                      int xMarks, int yMarks, Color c)
   {
      this(xMin, xMax, yMin, yMax, 0.0, xMarks, yMarks, c);
   }


   /**
       The z parameter is so that you can put the axis just above
       or just below the xy-plane (say z=0.01 or z=-0.01). This way,
       the axes can be just in front of or just behind whatever is
       being drawn in the xy-plane.
   */
   public Axes2DModel(double xMin, double xMax, double yMin, double yMax,
                      double z, int xMarks, int yMarks, Color c)
   {
      super();

      Primitive xAxis = new LineSegment(new Vertex(xMin, 0, z),
                                        new Vertex(xMax, 0, z));
      Primitive yAxis = new LineSegment(new Vertex(0, yMin, z),
                                        new Vertex(0, yMax, z));
      this.addPrimitive( xAxis );
      this.addPrimitive( yAxis );

       // Put evenly spaced tick marks on the x-axis.
      double xDelta = (xMax - xMin)/xMarks;
      double yDelta = (yMax - yMin)/50;
      for (double x = xMin; x <= xMax; x += xDelta)
      {
         Primitive ls = new LineSegment(new Vertex(x,  yDelta/2, z),
                                        new Vertex(x, -yDelta/2, z));
         this.addPrimitive( ls );
      }

       // Put evenly spaced tick marks on the y-axis.
      yDelta = (yMax - yMin)/yMarks;
      xDelta = (xMax - xMin)/50;
      for (double y = yMin; y <= yMax; y += yDelta)
      {
         Primitive ls = new LineSegment(new Vertex( xDelta/2, y, z),
                                        new Vertex(-xDelta/2, y, z));
         this.addPrimitive( ls );
      }

      this.setColor( c );
   }
}