/*

*/
package scene.models3;
import scene.*;
import scene.primitives.*;

/**
                  v[3]
                   +-----------------+ v[2]
                  /|                /|
                /  |              /  |
              /    |            /    |
            /      |          /      |
      v[4] +-----------------+ v[5]  |
           |       |         |       |               y
           |       |         |       |               |
           |       |         |       |               |
           |  v[0] +---------|-------+ v[1]          |
           |      /          |      /                |
           |    /            |    /                  +----. x
           |  /              |  /                   /
           |/                |/                    /
           +-----------------+                    /
          v[7]              v[6]                 z

   xs the size of the cube along the x-axis
   ys the size of the cube along the y-axis
   zs the size of the cube along the z-axis
*/
public class CubeModel extends Model
{
   public CubeModel(double xs, double ys, double zs)
   {
      super();

      // Create the cube's geometry.
      Vertex[] v = new Vertex[8];
      v[0] = new Vertex(0,    0,    0);
      v[1] = new Vertex(0+xs, 0,    0);
      v[2] = new Vertex(0+xs, 0+ys, 0);
      v[3] = new Vertex(0,    0+ys, 0);
      v[4] = new Vertex(0,    0+ys, 0+zs);
      v[5] = new Vertex(0+xs, 0+ys, 0+zs);
      v[6] = new Vertex(0+xs, 0,    0+zs);
      v[7] = new Vertex(0,    0,    0+zs);

      // Create six square faces.
      // Make sure the faces are all
      // oriented in the same way!
      // Remember that front facing
      // should be counter-clockwise
      // (and back facing is clock-wise).

      // front face
      Primitive face = new Face();
      face.addVertex( v[5] );
      face.addVertex( v[4] );
      face.addVertex( v[7] );
      face.addVertex( v[6] );
      this.addPrimitive( face );

      // top face
      face = new Face();
      face.addVertex( v[5] );
      face.addVertex( v[2] );
      face.addVertex( v[3] );
      face.addVertex( v[4] );
      this.addPrimitive( face );

      // right side
      face = new Face();
      face.addVertex( v[5] );
      face.addVertex( v[6] );
      face.addVertex( v[1] );
      face.addVertex( v[2] );
      this.addPrimitive( face );

      // back face
      face = new Face();
      face.addVertex( v[0] );
      face.addVertex( v[3] );
      face.addVertex( v[2] );
      face.addVertex( v[1] );
      this.addPrimitive( face );

      // bottom face
      face = new Face();
      face.addVertex( v[0] );
      face.addVertex( v[1] );
      face.addVertex( v[6] );
      face.addVertex( v[7] );
      this.addPrimitive( face );

      // left side
      face = new Face();
      face.addVertex( v[0] );
      face.addVertex( v[7] );
      face.addVertex( v[4] );
      face.addVertex( v[3] );
      this.addPrimitive( face );
   }
}//CubeModel