/*

*/
package scene.models3;
import scene.*;
import scene.primitives.*;

/**
   k   number of lines of longitude
   n+2 number of lines of latitude (there is always at least two)

   r   radius of the cylinder
   h   height of the cylinder
*/
public class CylinderModel extends Model
{
   public CylinderModel(int k, int n, double radius, double h)
   {
      super();

      // Create the cylinder's geometry.

      // An array of vertices to be used to create geometric primitives.
      Vertex[][] v = new Vertex[n+2][k];

      // Create all the vertices along the cylinder wall.
      for (int j = 0; j < k; j++)
      {
         double c = Math.cos(j*(2.0*Math.PI)/k);
         double s = Math.sin(j*(2.0*Math.PI)/k);
         for (int i = 0; i <= n+1; i++)
         {
            v[i][j] = new Vertex( radius*c, h-i*((2.0*h)/(n+1)), radius*s );
         }
      }

      // Vertex at the center of the top.
      Vertex    topCenter = new Vertex(0.0,  h, 0.0);
      // Vertex at the center of the bottom.
      Vertex bottomCenter = new Vertex(0.0, -h, 0.0);


      // Create the triangle fan at the top.
      Face face;
      for (int j = 0; j < k-1; j++)
      {
         face = new Face();
         face.addVertex( topCenter );
         face.addVertex( v[0][j+1] );
         face.addVertex( v[0][j  ] );
         this.addPrimitive( face );
      }
      face = new Face();
      face.addVertex( topCenter );
      face.addVertex( v[0][0] );
      face.addVertex( v[0][k-1] );
      this.addPrimitive( face );


      // Create all the rectangle strips between the top and bottom triangle fans.
      for (int i = 0; i <= n; i++)
      {
         for (int j = 0; j < k-1; j++)
         {
            face = new Face();
            face.addVertex( v[i  ][j  ] );
            face.addVertex( v[i  ][j+1] );
            face.addVertex( v[i+1][j+1] );
            face.addVertex( v[i+1][j  ] );
            this.addPrimitive( face );
         }
         face = new Face();
         face.addVertex( v[i  ][k-1] );
         face.addVertex( v[i  ][0  ] );
         face.addVertex( v[i+1][0  ] );
         face.addVertex( v[i+1][k-1] );
         this.addPrimitive( face );
      }


      // Create the triangle fan at the bottom.
      for (int j = 0; j < k-1; j++)
      {
         face = new Face();
         face.addVertex( bottomCenter );
         face.addVertex( v[n][j  ] );
         face.addVertex( v[n][j+1] );
         this.addPrimitive( face );
      }
      face = new Face();
      face.addVertex( bottomCenter );
      face.addVertex( v[n][k-1] );
      face.addVertex( v[n][0  ] );
      this.addPrimitive( face );
   }
}//CylinderModel