/*

*/
package scene.models3;
import scene.*;
import scene.primitives.*;

/**
   k   number of lines of longitude
   n+2 number of lines of latitude (there is always at least two)
   r1  radius of the base of the frustum
   r2  radius of the top of the frustum
   h   height of the frustum
*/
public class FrustumCircularModel extends Model
{
   public FrustumCircularModel(int k, int n, double r1, double r2, double h)
   {
      super();

      // Create the frustum's geometry.

      // An array of vertices to be used to create geometric primitives.
      Vertex[][] v = new Vertex[n+2][k];

      // Create all the vertices along the frustum wall.
      for (int j = 0; j < k; j++)
      {
         double c = Math.cos(j*(2.0*Math.PI)/k);
         double s = Math.sin(j*(2.0*Math.PI)/k);
         for (int i = 0; i <= n+1; i++)
         {
            double slantRadius = (i/(double)(n+1)) * r1 + ((n+1-i)/(double)(n+1)) * r2;
            v[i][j] = new Vertex( slantRadius*c, h-(i*h)/(n+1), slantRadius*s );
         }
      }

      // Vertex at the center of the top.
      Vertex    topCenter = new Vertex(0.0,   h, 0.0);
      // Vertex at the center of the bottom.
      Vertex bottomCenter = new Vertex(0.0, 0.0, 0.0);


      // Create the triangle fan at the top.
      Face face;
      for (int j = 0; j < k-1; j++)
      {
         face = new Face();
         face.addVertex( topCenter );
         face.addVertex( v[0][j+1] );
         face.addVertex( v[0][j  ] );
         this.addPrimitive( face );
      }
      face = new Face();
      face.addVertex( topCenter );
      face.addVertex( v[0][0] );
      face.addVertex( v[0][k-1] );
      this.addPrimitive( face );


      // Create all the rectangle strips between the top and bottom triangle fans.
      for (int i = 0; i <= n; i++)
      {
         for (int j = 0; j < k-1; j++)
         {
            face = new Face();
            face.addVertex( v[i  ][j  ] );
            face.addVertex( v[i  ][j+1] );
            face.addVertex( v[i+1][j+1] );
            face.addVertex( v[i+1][j  ] );
            this.addPrimitive( face );
         }
         face = new Face();
         face.addVertex( v[i  ][k-1] );
         face.addVertex( v[i  ][0  ] );
         face.addVertex( v[i+1][0  ] );
         face.addVertex( v[i+1][k-1] );
         this.addPrimitive( face );
      }


      // Create the triangle fan at the bottom.
      for (int j = 0; j < k-1; j++)
      {
         face = new Face();
         face.addVertex( bottomCenter );
         face.addVertex( v[n][j  ] );
         face.addVertex( v[n][j+1] );
         this.addPrimitive( face );
      }
      face = new Face();
      face.addVertex( bottomCenter );
      face.addVertex( v[n][k-1] );
      face.addVertex( v[n][0  ] );
      this.addPrimitive( face );
   }
}//FrustumCircularModel