/*

*/
package scene.models3;
import scene.*;
import scene.primitives.*;

/**
   This is a regular octahedron with side length 1.
   Its center is at the orgin and the center plane is the xz-plane.
   The 5th and 6th vertices are on the y-axis.

   http://en.wikipedia.org/wiki/Regular_octahedron
*/
public class OctahedronModel extends Model
{
   public OctahedronModel()
   {
      super();

      // Create the octahedron's geometry.
      double sqrt3 = Math.sqrt(3.0);
      double sqrt2 = Math.sqrt(2.0);
      Vertex[] v = new Vertex[6];
      v[0] = new Vertex( 0.5, 0,  0.5);  // four vertices around the center plane
      v[1] = new Vertex(-0.5, 0,  0.5);
      v[2] = new Vertex(-0.5, 0, -0.5);
      v[3] = new Vertex( 0.5, 0, -0.5);
      v[4] = new Vertex( 0,  1/sqrt2, 0);   // vertex at the top
      v[5] = new Vertex( 0, -1/sqrt2, 0);   // vertex at the bottom

      // Create eight triangular faces.
      // Make sure the faces are all
      // oriented in the same way!
      // Remember that front facing
      // should be counter-clockwise
      // (and back facing is clock-wise).

      Primitive face = new Face();
      face.addVertex( v[4] );
      face.addVertex( v[1] );
      face.addVertex( v[0] );
      this.addPrimitive( face );

      face = new Face();
      face.addVertex( v[4] );
      face.addVertex( v[0] );
      face.addVertex( v[3] );
      this.addPrimitive( face );

      face = new Face();
      face.addVertex( v[4] );
      face.addVertex( v[2] );
      face.addVertex( v[1] );
      this.addPrimitive( face );

      face = new Face();
      face.addVertex( v[4] );
      face.addVertex( v[3] );
      face.addVertex( v[2] );
      this.addPrimitive( face );

      face = new Face();
      face.addVertex( v[5] );
      face.addVertex( v[0] );
      face.addVertex( v[1] );
      this.addPrimitive( face );

      face = new Face();
      face.addVertex( v[5] );
      face.addVertex( v[3] );
      face.addVertex( v[0] );
      this.addPrimitive( face );

      face = new Face();
      face.addVertex( v[5] );
      face.addVertex( v[1] );
      face.addVertex( v[2] );
      this.addPrimitive( face );

      face = new Face();
      face.addVertex( v[5] );
      face.addVertex( v[2] );
      face.addVertex( v[3] );
      this.addPrimitive( face );
   }
}//OctahedronModel