/*

*/
package scene.models3;
import scene.*;
import scene.primitives.*;

/**
   k   number of lines of longitude
   n+1 number of lines of latitude (there is always at least one)

   r radius of the sphere

*/
public class SphereModel extends Model
{
   public SphereModel(int k, int n, double radius)
   {
      super();

      // Create the sphere's geometry.

      // An array of vertices to be used to create geometric primitives.
      Vertex[][] v = new Vertex[n+1][k];

      // Create all the vertices.
      for (int j = 0; j < k; j++)
      {
         double c1 = Math.cos(j*(2.0*Math.PI)/k);
         double s1 = Math.sin(j*(2.0*Math.PI)/k);
         for (int i = 0; i <= n; i++)
         {
            double c2 = Math.cos((i+1)*Math.PI/(n+2));
            double s2 = Math.sin((i+1)*Math.PI/(n+2));
            v[i][j] = new Vertex( radius*s2*c1, radius*c2, radius*s2*s1 );
         }
      }
      // Vertex at the center of the top.
      Vertex    topPole = new Vertex(0.0,  radius, 0.0);
      // Vertex at the center of the bottom.
      Vertex bottomPole = new Vertex(0.0, -radius, 0.0);

      // Create the triangle fan at the top.
      Face face;
      for (int j = 0; j < k-1; j++)
      {
         face = new Face();
         face.addVertex( topPole );
         face.addVertex( v[0][j+1] );
         face.addVertex( v[0][j  ] );
         this.addPrimitive( face );
      }
      face = new Face();
      face.addVertex( topPole );
      face.addVertex( v[0][0] );
      face.addVertex( v[0][k-1] );
      this.addPrimitive( face );


      // Create all the rectangle strips between the top and bottom triangle fans.
      for (int i = 0; i < n; i++)
      {
         for (int j = 0; j < k-1; j++)
         {
            face = new Face();
            face.addVertex( v[i  ][j  ] );
            face.addVertex( v[i  ][j+1] );
            face.addVertex( v[i+1][j+1] );
            face.addVertex( v[i+1][j  ] );
            this.addPrimitive( face );
         }
         face = new Face();
         face.addVertex( v[i  ][k-1] );
         face.addVertex( v[i  ][0  ] );
         face.addVertex( v[i+1][0  ] );
         face.addVertex( v[i+1][k-1] );
         this.addPrimitive( face );
      }


      // Create the triangle fan at the bottom.
      for (int j = 0; j < k-1; j++)
      {
         face = new Face();
         face.addVertex( bottomPole );
         face.addVertex( v[n][j  ] );
         face.addVertex( v[n][j+1] );
         this.addPrimitive( face );
      }
      face = new Face();
      face.addVertex( bottomPole );
      face.addVertex( v[n][k-1] );
      face.addVertex( v[n][0  ] );
      this.addPrimitive( face );
   }
}//SphereModel