/*

*/
package scene.models3;
import scene.*;
import scene.primitives.*;

/**
   This is a regular tetrahedron with side length 1.
   One face is in the xz-plane, with its center at the orgin.
   The 4th vertex in on the positive y-axis.

   http://en.wikipedia.org/wiki/Tetrahedron


                     (1, 0, -1/sqrt(2)) (0, 1, 1/sqrt(2)) edge length 2

                     (sqrt(2), 0, -1)   (0, sqrt(2), 1)   edge length 2*sqrt(2)

                     ( 0, 0, 0)  ( 1, 1, 0)  ( 0, 1, 1)  ( 1, 0, 1)  edge length sqrt(2)
        Tetrahedron: ( 1,-1,-1)  (-1, 1,-1)  ( 1, 1, 1)  (-1,-1, 1)  edge length 2*sqrt(2)
   Dual Tetrahedron: (-1, 1, 1)  ( 1,-1, 1)  (-1,-1,-1)  ( 1, 1,-1)
*/
public class TetrahedronModel extends Model
{
   public TetrahedronModel()
   {
      super();

      // Create the tetrahedron's geometry.
      double sqrt3 = Math.sqrt(3.0);
      double sqrt2 = Math.sqrt(2.0);
      Vertex[] v = new Vertex[4];
      v[0] = new Vertex( 1/sqrt3,     0,  0);    // three vertices around the bottom face
      v[1] = new Vertex(-1/(2*sqrt3), 0,  0.5);
      v[2] = new Vertex(-1/(2*sqrt3), 0, -0.5);
      v[3] = new Vertex( 0, sqrt2/sqrt3,  0);    // vertex at the top

      // Create four triangular faces.
      // Make sure the faces are all
      // oriented in the same way!
      // Remember that front facing
      // should be counter-clockwise
      // (and back facing is clock-wise).

      Primitive face = new Face();
      face.addVertex( v[3] );
      face.addVertex( v[1] );
      face.addVertex( v[0] );
      this.addPrimitive( face );

      face = new Face();
      face.addVertex( v[3] );
      face.addVertex( v[2] );
      face.addVertex( v[1] );
      this.addPrimitive( face );

      face = new Face();
      face.addVertex( v[3] );
      face.addVertex( v[0] );
      face.addVertex( v[2] );
      this.addPrimitive( face );

      face = new Face();
      face.addVertex( v[0] );
      face.addVertex( v[1] );
      face.addVertex( v[2] );
      this.addPrimitive( face );
   }
}//TetrahedronModel