/*

*/
package scene.primitives;
import scene.*;

import java.util.LinkedList;
import java.util.ArrayList;
import java.util.List;

/**
   A LineSegment object has two Vertex objects that
   represent the endpoints of the line segment.
<p>
   A LineSegment knows how to assemble itself into a list
   of LineSegment objects, so we can implementation the
   Primitive interface.
*/
public class LineSegment extends Primitive
{
   /**
      Construct a default (empty) LineSegment.
   */
   public LineSegment()
   {
      super();
   }


   /**
      Construct a LineSegment from an array of vertices.

      @param vArray array of vertices to put into the new LineSegment
   */
   public LineSegment(Vertex... vArray)
   {
      super(vArray);
   }


   /**
      Construct a LineSegment from a List of vertices.

      @param vList list of vertices to put into the new LineSegment
   */
   public LineSegment(List<Vertex> vList)
   {
      super(vList);
   }


   /**
      Create a LineSegment object with references to the two given Vertex objects.

      @param v0 1st endpoint of the new LineSegment
      @param v1 2nd endpoint of the new LineSegment
   */
   public LineSegment(Vertex v0, Vertex v1)
   {
      super();

      this.vertexList.add( v0 );
      this.vertexList.add( v1 );
   }


   /**
      Create a LineSegment object with references to copies of the two
      Vertex objects in the given LineSegment object.
      <p>
      Notice that this is a "deep copy" of the given LineSegment object.

      @param ls LineSegment to make a deep copy of
   */
   public LineSegment(LineSegment ls) // a "copy constructor"
   {
      super();

      this.vertexList.add( new Vertex(ls.vertexList.get(0)) );
      this.vertexList.add( new Vertex(ls.vertexList.get(1)) );
   }


   /**
      Implement the makeCopy() method from the abstract base class.
      <p>
      Make a deep copy of this LineSegment primitive.

      @return a LineSegment object that is a deep copy of this LineSegment
   */
   public Primitive makeCopy()
   {
      return new LineSegment( new Vertex(this.vertexList.get(0)),
                              new Vertex(this.vertexList.get(1)) );
   }


   /**
      Implement the assemble() method from the abstract base class.
      <p>
      A LineSegment should return a List<LineSegment>
      object containing just this one LineSgement object.

      @return a List of the LineSegment object that represents this LineSegment
   */
   public List<LineSegment> assemble()
   {
    //List<LineSegment> result = new LinkedList<LineSegment>();
      List<LineSegment> result = new ArrayList<LineSegment>(1);
      result.add( this );
      return result;
   }


    /**
      For debugging.

      @return String representation of this LineSegment object
    */
    public String toString()
    {
       String result = "Line Segment:\n";
       result += this.vertexList.get(0).toString();
       result += this.vertexList.get(1).toString();
       return result;
   }
}
